/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* Intrinsic Function Source Fragment                                         *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/

#ifndef LLVM_IR_INTRINSIC_SPV_ENUMS_H
#define LLVM_IR_INTRINSIC_SPV_ENUMS_H

namespace llvm {
namespace Intrinsic {
enum SPVIntrinsics : unsigned {
// Enum values for intrinsics
    spv_alloca = 10804,                                // llvm.spv.alloca
    spv_assign_name,                           // llvm.spv.assign.name
    spv_assign_ptr_type,                       // llvm.spv.assign.ptr.type
    spv_assign_type,                           // llvm.spv.assign.type
    spv_assume,                                // llvm.spv.assume
    spv_bitcast,                               // llvm.spv.bitcast
    spv_cmpxchg,                               // llvm.spv.cmpxchg
    spv_const_composite,                       // llvm.spv.const.composite
    spv_expect,                                // llvm.spv.expect
    spv_extractelt,                            // llvm.spv.extractelt
    spv_extractv,                              // llvm.spv.extractv
    spv_gep,                                   // llvm.spv.gep
    spv_init_global,                           // llvm.spv.init.global
    spv_insertelt,                             // llvm.spv.insertelt
    spv_insertv,                               // llvm.spv.insertv
    spv_load,                                  // llvm.spv.load
    spv_ptrcast,                               // llvm.spv.ptrcast
    spv_store,                                 // llvm.spv.store
    spv_switch,                                // llvm.spv.switch
    spv_track_constant,                        // llvm.spv.track.constant
    spv_undef,                                 // llvm.spv.undef
    spv_unreachable,                           // llvm.spv.unreachable
    spv_unref_global,                          // llvm.spv.unref.global
}; // enum
} // namespace Intrinsic
} // namespace llvm

#endif
